=== modified file 'NEWS'
--- NEWS	2006-09-12 06:22:31 +0000
+++ NEWS	2006-09-12 06:47:51 +0000
@@ -8,6 +8,7 @@
       requests. (Andrew Bennetts, Robert Collins, Martin Pool)
 
   BUG FIXES:
+
     * revert now removes newly-added directories (Aaron Bentley, #54172)
 
     * ``bzr upgrade sftp://`` shouldn't fail to upgrade v6 branches if there 
@@ -22,6 +23,10 @@
       return urlescaped paths. This is now tested (there were bugs in a few
       of the transports) (Andrew Bennetts, David Allouche, John Arbash Meinel)
 
+    * Add a new method ``Transport.get_smart_client()``. This is provided to
+      allow upgrades to a richer interface than the VFS one provided by
+      Transport. (Andrew Bennetts, Martin Pool)
+
 bzr 0.10.0RC1  2006-08-28
   
   IMPROVEMENTS:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-08-30 02:33:36 +0000
+++ bzrlib/errors.py	2006-09-12 06:47:51 +0000
@@ -1115,6 +1115,13 @@
         BzrNewError.__init__(self)    
 
 
+class NoSmartServer(NotBranchError):
+    """No smart server available at %(url)s"""
+
+    def __init__(self, url):
+        self.url = url
+
+
 class UnknownSSH(BzrNewError):
     """Unrecognised value for BZR_SSH environment variable: %(vendor)s"""
 

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-08-30 06:34:50 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-09-12 06:47:51 +0000
@@ -30,13 +30,13 @@
     urlutils,
     )
 from bzrlib.errors import (DirectoryNotEmpty, NoSuchFile, FileExists,
-                           LockError, PathError,
+                           LockError, NoSmartServer, PathError,
                            TransportNotPossible, ConnectionError,
                            InvalidURL)
 from bzrlib.osutils import getcwd
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
 from bzrlib.tests.test_transport import TestTransportImplementation
-from bzrlib.transport import memory
+from bzrlib.transport import memory, smart
 import bzrlib.transport
 
 
@@ -1074,3 +1074,18 @@
         self.assertEqual(d[2], (0, '0'))
         self.assertEqual(d[3], (3, '34'))
 
+    def test_get_smart_client(self):
+        """All transports must either give a smart client, or know they can't.
+
+        For some transports such as http this might depend on probing to see 
+        what's actually present on the other end.  (But we can adjust for that 
+        in the future.)
+        """
+        transport = self.get_transport()
+        try:
+            client = transport.get_smart_client()
+            # XXX: should be a more general class
+            self.assertIsInstance(client, smart.SmartStreamClient)
+        except NoSmartServer:
+            # as long as we got it we're fine
+            pass

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-09-12 06:22:31 +0000
+++ bzrlib/transport/__init__.py	2006-09-12 06:47:51 +0000
@@ -420,9 +420,17 @@
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
+        :rtype: File-like object.
         """
         raise NotImplementedError(self.get)
 
+    def get_smart_client(self):
+        """Return a smart client for this transport if possible.
+
+        :raises NoSmartServer: if no smart server client is available.
+        """
+        raise errors.NoSmartServer(self.base)
+
     def readv(self, relpath, offsets):
         """Get parts of the file at the given relative path.
 

